@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 1998-2016 A. S. Hodel
+Copyright (C) 1998-2018 A. S. Hodel
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -32,14 +32,18 @@ along with Octave; see the file COPYING.  If not, see
 #  include "config.h"
 #endif
 
-#include <cfloat>
+#include <cctype>
+#include <cmath>
 
-#include <iostream>
-#include <iomanip>
+#if defined (DEBUG) || defined (DEBUG_SORT) || defined (DEBUG_EIG)
+#  include <iostream>
+#  if defined (DEBUG_EIG)
+#    include <iomanip>
+#  endif
+#endif
 
 #include "f77-fcn.h"
 #include "lo-lapack-proto.h"
-#include "lo-math.h"
 #include "qr.h"
 #include "quit.h"
 
@@ -47,31 +51,23 @@ along with Octave; see the file COPYING.  If not, see
 #include "error.h"
 #include "errwarn.h"
 #include "ovl.h"
-#include "oct-map.h"
-#include "ov.h"
-#include "pager.h"
 #if defined (DEBUG) || defined (DEBUG_SORT)
 #  include "pr-output.h"
 #endif
-#include "symtab.h"
-#include "utils.h"
-#include "variables.h"
 
-typedef octave_idx_type (*sort_function) (const octave_idx_type& LSIZE,
-                                          const double& ALPHA,
-                                          const double& BETA, const double& S,
-                                          const double& P);
+typedef F77_INT (*sort_function) (const F77_INT& LSIZE,
+                                  const double& ALPHA, const double& BETA,
+                                  const double& S, const double& P);
 
 extern "C"
 {
-  // Van Dooren's code (netlib.org: toms/590) for reordering
-  // GEP.  Only processes Z, not Q.
+  // Van Dooren's code (netlib.org: toms/590) for reordering GEP.
+  // Only processes Z, not Q.
   F77_RET_T
-  F77_FUNC (dsubsp, DSUBSP) (const F77_INT& NMAX,
-                             const F77_INT& N, F77_DBLE* A,
-                             F77_DBLE* B, F77_DBLE* Z, sort_function,
-                             const F77_DBLE& EPS, F77_INT& NDIM,
-                             F77_INT& FAIL, F77_INT* IND);
+  F77_FUNC (dsubsp, DSUBSP) (const F77_INT& NMAX, const F77_INT& N,
+                             F77_DBLE *A, F77_DBLE *B, F77_DBLE *Z,
+                             sort_function, const F77_DBLE& EPS,
+                             F77_INT& NDIM, F77_INT& FAIL, F77_INT *IND);
 }
 
 // fcrhp, fin, fout, folhp:
@@ -82,9 +78,9 @@ extern "C"
 //   fcrhp: real(lambda) >= 0
 //   folhp: real(lambda) < 0
 
-static octave_idx_type
-fcrhp (const octave_idx_type& lsize, const double& alpha,
-       const double& beta, const double& s, const double&)
+static F77_INT
+fcrhp (const F77_INT& lsize, const double& alpha, const double& beta,
+       const double& s, const double&)
 {
   if (lsize == 1)
     return (alpha * beta >= 0 ? 1 : -1);
@@ -92,16 +88,16 @@ fcrhp (const octave_idx_type& lsize, const double& alpha,
     return (s >= 0 ? 1 : -1);
 }
 
-static octave_idx_type
-fin (const octave_idx_type& lsize, const double& alpha,
-     const double& beta, const double&, const double& p)
+static F77_INT
+fin (const F77_INT& lsize, const double& alpha, const double& beta,
+     const double&, const double& p)
 {
-  octave_idx_type retval;
+  F77_INT retval;
 
   if (lsize == 1)
-    retval = (fabs (alpha) < fabs (beta) ? 1 : -1);
+    retval = (std::abs (alpha) < std::abs (beta) ? 1 : -1);
   else
-    retval = (fabs (p) < 1 ? 1 : -1);
+    retval = (std::abs (p) < 1 ? 1 : -1);
 
 #if defined (DEBUG)
   std::cout << "qz: fin: retval=" << retval << std::endl;
@@ -110,9 +106,9 @@ fin (const octave_idx_type& lsize, const double& alpha,
   return retval;
 }
 
-static octave_idx_type
-folhp (const octave_idx_type& lsize, const double& alpha,
-       const double& beta, const double& s, const double&)
+static F77_INT
+folhp (const F77_INT& lsize, const double& alpha, const double& beta,
+       const double& s, const double&)
 {
   if (lsize == 1)
     return (alpha * beta < 0 ? 1 : -1);
@@ -120,14 +116,14 @@ folhp (const octave_idx_type& lsize, const double& alpha,
     return (s < 0 ? 1 : -1);
 }
 
-static octave_idx_type
-fout (const octave_idx_type& lsize, const double& alpha,
-      const double& beta, const double&, const double& p)
+static F77_INT
+fout (const F77_INT& lsize, const double& alpha, const double& beta,
+      const double&, const double& p)
 {
   if (lsize == 1)
-    return (fabs (alpha) >= fabs (beta) ? 1 : -1);
+    return (std::abs (alpha) >= std::abs (beta) ? 1 : -1);
   else
-    return (fabs (p) >= 1 ? 1 : -1);
+    return (std::abs (p) >= 1 ? 1 : -1);
 }
 
 //FIXME: Matlab does not produce lambda as the first output argument.
@@ -135,27 +131,39 @@ fout (const octave_idx_type& lsize, const double& alpha,
 DEFUN (qz, args, nargout,
        doc: /* -*- texinfo -*-
 @deftypefn  {} {@var{lambda} =} qz (@var{A}, @var{B})
-@deftypefnx {} {@var{lambda} =} qz (@var{A}, @var{B}, @var{opt})
-QZ@tie{}decomposition of the generalized eigenvalue problem
-(@math{A x = s B x}).
+@deftypefnx {} {[@var{AA}, @var{BB}, @var{Q}, @var{Z}, @var{V}, @var{W}, @var{lambda}] =} qz (@var{A}, @var{B})
+@deftypefnx {} {[@var{AA}, @var{BB}, @var{Z}] =} qz (@var{A}, @var{B}, @var{opt})
+@deftypefnx {} {[@var{AA}, @var{BB}, @var{Z}, @var{lambda}] =} qz (@var{A}, @var{B}, @var{opt})
+Compute the QZ@tie{}decomposition of a generalized eigenvalue problem.
+
+The generalized eigenvalue problem is defined as
+
+@tex
+$$A x = \lambda B x$$
+@end tex
+@ifnottex
+
+@math{A x = @var{lambda} B x}
+
+@end ifnottex
+
+There are three calling forms of the function:
 
-There are three ways to call this function:
 @enumerate
 @item @code{@var{lambda} = qz (@var{A}, @var{B})}
 
-Computes the generalized eigenvalues
+Compute the generalized eigenvalues
 @tex
-$\lambda$
+$\lambda.$
 @end tex
 @ifnottex
-@var{lambda}
+@var{lambda}.
 @end ifnottex
-of @math{(A - s B)}.
 
-@item @code{[AA, BB, Q, Z, V, W, @var{lambda}] = qz (@var{A}, @var{B})}
+@item @code{[@var{AA}, @var{BB}, @var{Q}, @var{Z}, @var{V}, @var{W}, @var{lambda}] = qz (@var{A}, @var{B})}
 
-Computes QZ@tie{}decomposition, generalized eigenvectors, and generalized
-eigenvalues of @math{(A - s B)}
+Compute QZ@tie{}decomposition, generalized eigenvectors, and generalized
+eigenvalues.
 @tex
 $$ AV = BV{ \rm diag }(\lambda) $$
 $$ W^T A = { \rm diag }(\lambda)W^T B $$
@@ -166,20 +174,20 @@ $$ AA = Q^T AZ, BB = Q^T BZ $$
 @example
 @group
 
-A * V = B * V * diag (@var{lambda})
-W' * A = diag (@var{lambda}) * W' * B
-AA = Q * A * Z, BB = Q * B * Z
+@var{A} * @var{V} = @var{B} * @var{V} * diag (@var{lambda})
+@var{W}' * @var{A} = diag (@var{lambda}) * @var{W}' * @var{B}
+@var{AA} = @var{Q} * @var{A} * @var{Z}, @var{BB} = @var{Q} * @var{B} * @var{Z}
 
 @end group
 @end example
 
 @end ifnottex
-with @var{Q} and @var{Z} orthogonal (unitary)= @var{I}
+with @var{Q} and @var{Z} orthogonal (unitary for complex case).
 
-@item @code{[AA,BB,Z@{, @var{lambda}@}] = qz (@var{A}, @var{B}, @var{opt})}
+@item @code{[@var{AA}, @var{BB}, @var{Z} @{, @var{lambda}@}] = qz (@var{A}, @var{B}, @var{opt})}
 
-As in form [2], but allows ordering of generalized eigenpairs for, e.g.,
-solution of discrete time algebraic Riccati equations.  Form 3 is not
+As in form 2 above, but allows ordering of generalized eigenpairs for, e.g.,
+solution of discrete time algebraic @nospell{Riccati} equations.  Form 3 is not
 available for complex matrices, and does not compute the generalized
 eigenvectors @var{V}, @var{W}, nor the orthogonal matrix @var{Q}.
 
@@ -190,32 +198,45 @@ the revised pencil contains all eigenvalues that satisfy:
 
 @table @asis
 @item @qcode{"N"}
-= unordered (default)
+unordered (default)
 
 @item @qcode{"S"}
-= small: leading block has all |lambda| @leq{} 1
+small: leading block has all
+@tex
+$|\lambda| < 1$
+@end tex
+@ifnottex
+|@var{lambda}| < 1
+@end ifnottex
 
 @item @qcode{"B"}
-= big: leading block has all |lambda| @geq{} 1
+big: leading block has all
+@tex
+$|\lambda| \geq 1$
+@end tex
+@ifnottex
+|@var{lambda}| @geq{} 1
+@end ifnottex
 
 @item @qcode{"-"}
-= negative real part: leading block has all eigenvalues
-in the open left half-plane
+negative real part: leading block has all eigenvalues in the open left
+half-plane
 
 @item @qcode{"+"}
-= non-negative real part: leading block has all eigenvalues
-in the closed right half-plane
+non-negative real part: leading block has all eigenvalues in the closed right
+half-plane
 @end table
 @end table
 @end enumerate
 
 Note: @code{qz} performs permutation balancing, but not scaling
-(@pxref{XREFbalance}).  The order of output arguments was selected for
-compatibility with @sc{matlab}.
+(@pxref{XREFbalance,,balance}), which may be lead to less accurate results than
+@code{eig}.  The order of output arguments was selected for compatibility with
+@sc{matlab}.
 @seealso{eig, balance, lu, chol, hess, qr, qzhess, schur, svd}
 @end deftypefn */)
 {
-  volatile int nargin = args.length ();
+  int nargin = args.length ();
 
 #if defined (DEBUG)
   std::cout << "qz: nargin = " << nargin
@@ -226,30 +247,30 @@ compatibility with @sc{matlab}.
     print_usage ();
 
   if (nargin == 3 && (nargout < 3 || nargout > 4))
-    error ("qz: invalid number of output arguments for form [3] call");
+    error ("qz: invalid number of output arguments for form 3 call");
 
 #if defined (DEBUG)
   std::cout << "qz: determine ordering option" << std::endl;
 #endif
 
   // Determine ordering option.
-  volatile char ord_job = 0;
-  static double safmin;
+  // declared volatile to avoid compiler warnings about long jumps vforks.
+  volatile char ord_job = 'N';
+  double safmin = 0.0;
 
-  if (nargin == 2)
-    ord_job = 'N';
-  else
+  if (nargin == 3)
     {
-      std::string tmp = args(2).xstring_value ("qz: OPT must be a string");
+      std::string opt = args(2).xstring_value ("qz: OPT must be a string");
 
-      if (! tmp.empty ())
-        ord_job = tmp[0];
+      if (opt.empty ())
+        error ("qz: OPT must be a non-empty string");
 
-      if (! (ord_job == 'N' || ord_job == 'n'
-             || ord_job == 'S' || ord_job == 's'
-             || ord_job == 'B' || ord_job == 'b'
-             || ord_job == '+' || ord_job == '-'))
-        error ("qz: invalid order option");
+      ord_job = std::toupper (opt[0]);
+
+      std::string valid_opts = "NSB+-";
+
+      if (valid_opts.find_first_of (ord_job) == std::string::npos)
+        error ("qz: invalid order option '%c'", ord_job);
 
       // overflow constant required by dlag2
       F77_FUNC (xdlamch, XDLAMCH) (F77_CONST_CHAR_ARG2 ("S", 1),
@@ -283,80 +304,86 @@ compatibility with @sc{matlab}.
     }
 
 #if defined (DEBUG)
-  std::cout << "qz: check argument 1" << std::endl;
+  std::cout << "qz: check matrix A" << std::endl;
 #endif
 
-  // Argument 1: check if it's okay dimensioned.
-  octave_idx_type nn = args(0).rows ();
+  // Matrix A: check dimensions.
+  F77_INT nn = octave::to_f77_int (args(0).rows ());
+  F77_INT nc = octave::to_f77_int (args(0).columns ());
 
 #if defined (DEBUG)
-  std::cout << "argument 1 dimensions: ("
-            << nn << "," << args(0).columns () << ")"
-            << std::endl;
+  std::cout << "Matrix A dimensions: (" << nn << ',' << nc << ')' << std::endl;
 #endif
 
-  octave_value_list retval;
-
-  if (args(0).is_empty ())
+  if (args(0).isempty ())
     {
-      warn_empty_arg ("qz: parameter 1; continuing");
+      warn_empty_arg ("qz: A");
       return octave_value_list (2, Matrix ());
     }
-  else if (args(0).columns () != nn)
+  else if (nc != nn)
     err_square_matrix_required ("qz", "A");
 
-  // Argument 1: dimensions look good; get the value.
+  // Matrix A: get value.
   Matrix aa;
   ComplexMatrix caa;
 
-  if (args(0).is_complex_type ())
+  if (args(0).iscomplex ())
     caa = args(0).complex_matrix_value ();
   else
     aa = args(0).matrix_value ();
 
 #if defined (DEBUG)
-  std::cout << "qz: check argument 2" << std::endl;
+  std::cout << "qz: check matrix B" << std::endl;
 #endif
 
   // Extract argument 2 (bb, or cbb if complex).
-  if ((nn != args(1).columns ()) || (nn != args(1).rows ()))
+  F77_INT b_nr = octave::to_f77_int (args(1).rows ());
+  F77_INT b_nc = octave::to_f77_int (args(1).columns ());
+
+  if (nn != b_nc || nn != b_nr)
     err_nonconformant ();
 
   Matrix bb;
   ComplexMatrix cbb;
 
-  if (args(1).is_complex_type ())
+  if (args(1).iscomplex ())
     cbb = args(1).complex_matrix_value ();
   else
     bb = args(1).matrix_value ();
 
-  // Both matrices loaded, now let's check what kind of arithmetic:
-  // declared volatile to avoid compiler warnings about long jumps,
-  // vforks.
+  // Both matrices loaded, now check whether to calculate complex or real val.
 
-  volatile int complex_case
-    = (args(0).is_complex_type () || args(1).is_complex_type ());
+  // declared volatile to avoid compiler warnings about long jumps vforks.
+  volatile bool complex_case
+    = (args(0).iscomplex () || args(1).iscomplex ());
 
   if (nargin == 3 && complex_case)
     error ("qz: cannot re-order complex qz decomposition");
 
-  // First, declare variables used in both the real and complex case.
-  Matrix QQ(nn,nn), ZZ(nn,nn), VR(nn,nn), VL(nn,nn);
-  RowVector alphar(nn), alphai(nn), betar(nn);
-  ComplexRowVector xalpha(nn), xbeta(nn);
-  ComplexMatrix CQ(nn,nn), CZ(nn,nn), CVR(nn,nn), CVL(nn,nn);
-  octave_idx_type ilo, ihi, info;
-  char compq = (nargout >= 3 ? 'V' : 'N');
-  char compz = ((nargout >= 4 || nargin == 3)? 'V' : 'N');
+  // First, declare variables used in both the real and complex cases.
+  // FIXME: There are a lot of excess variables declared.
+  //        Probably a better way to handle this.
+  Matrix QQ (nn,nn), ZZ (nn,nn), VR (nn,nn), VL (nn,nn);
+  RowVector alphar (nn), alphai (nn), betar (nn);
+  ComplexRowVector xalpha (nn), xbeta (nn);
+  ComplexMatrix CQ (nn,nn), CZ (nn,nn), CVR (nn,nn), CVL (nn,nn);
+  F77_INT ilo, ihi, info;
+  char comp_q = (nargout >= 3 ? 'V' : 'N');
+  char comp_z = ((nargout >= 4 || nargin == 3)? 'V' : 'N');
 
-  // Initialize Q, Z to identity if we need either of them.
-  if (compq == 'V' || compz == 'V')
-    for (octave_idx_type ii = 0; ii < nn; ii++)
-      for (octave_idx_type jj = 0; jj < nn; jj++)
+  // Initialize Q, Z to identity matrix if either is needed
+  if (comp_q == 'V' || comp_z == 'V')
+    {
+      double *QQptr = QQ.fortran_vec ();
+      double *ZZptr = ZZ.fortran_vec ();
+      std::fill_n (QQptr, QQ.numel (), 0.0);
+      std::fill_n (ZZptr, ZZ.numel (), 0.0);
+      for (F77_INT i = 0; i < nn; i++)
         {
-          OCTAVE_QUIT;
-          QQ(ii,jj) = ZZ(ii,jj) = (ii == jj ? 1.0 : 0.0);
+          QQ(i,i) = 1.0;
+          ZZ(i,i) = 1.0;
         }
+    }
 
   // Always perform permutation balancing.
   const char bal_job = 'P';
@@ -365,20 +392,20 @@ compatibility with @sc{matlab}.
   if (complex_case)
     {
 #if defined (DEBUG)
-      if (compq == 'V')
+      if (comp_q == 'V')
         std::cout << "qz: performing balancing; CQ=" << std::endl
                   << CQ << std::endl;
 #endif
-      if (args(0).is_real_type ())
+      if (args(0).isreal ())
         caa = ComplexMatrix (aa);
 
-      if (args(1).is_real_type ())
+      if (args(1).isreal ())
         cbb = ComplexMatrix (bb);
 
-      if (compq == 'V')
+      if (comp_q == 'V')
         CQ = ComplexMatrix (QQ);
 
-      if (compz == 'V')
+      if (comp_z == 'V')
         CZ = ComplexMatrix (ZZ);
 
       F77_XFCN (zggbal, ZGGBAL,
@@ -392,7 +419,7 @@ compatibility with @sc{matlab}.
   else
     {
 #if defined (DEBUG)
-      if (compq == 'V')
+      if (comp_q == 'V')
         std::cout << "qz: performing balancing; QQ=" << std::endl
                   << QQ << std::endl;
 #endif
@@ -405,11 +432,13 @@ compatibility with @sc{matlab}.
                  F77_CHAR_ARG_LEN (1)));
     }
 
+  // Only permutation balance above is done.  Skip scaling balance.
+
+#if 0
   // Since we just want the balancing matrices, we can use dggbal
   // for both the real and complex cases; left first
 
-#if 0
-  if (compq == 'V')
+  if (comp_q == 'V')
     {
       F77_XFCN (dggbak, DGGBAK,
                 (F77_CONST_CHAR_ARG2 (&bal_job, 1),
@@ -420,13 +449,13 @@ compatibility with @sc{matlab}.
                  F77_CHAR_ARG_LEN (1)));
 
 #if defined (DEBUG)
-      if (compq == 'V')
+      if (comp_q == 'V')
         std::cout << "qz: balancing done; QQ=" << std::endl << QQ << std::endl;
 #endif
-  }
+    }
 
   // then right
-  if (compz == 'V')
+  if (comp_z == 'V')
     {
       F77_XFCN (dggbak, DGGBAK,
                 (F77_CONST_CHAR_ARG2 (&bal_job, 1),
@@ -437,14 +466,13 @@ compatibility with @sc{matlab}.
                  F77_CHAR_ARG_LEN (1)));
 
 #if defined (DEBUG)
-      if (compz == 'V')
+      if (comp_z == 'V')
         std::cout << "qz: balancing done; ZZ=" << std::endl << ZZ << std::endl;
 #endif
     }
 #endif
 
-  static char qz_job;
-  qz_job = (nargout < 2 ? 'E' : 'S');
+  char qz_job = (nargout < 2 ? 'E' : 'S');
 
   if (complex_case)
     {
@@ -459,8 +487,8 @@ compatibility with @sc{matlab}.
       CQ = CQ * cbqr.Q ();
 
       F77_XFCN (zgghrd, ZGGHRD,
-                (F77_CONST_CHAR_ARG2 (&compq, 1),
-                 F77_CONST_CHAR_ARG2 (&compz, 1),
+                (F77_CONST_CHAR_ARG2 (&comp_q, 1),
+                 F77_CONST_CHAR_ARG2 (&comp_z, 1),
                  nn, ilo, ihi, F77_DBLE_CMPLX_ARG (caa.fortran_vec ()),
                  nn, F77_DBLE_CMPLX_ARG (cbb.fortran_vec ()), nn,
                  F77_DBLE_CMPLX_ARG (CQ.fortran_vec ()), nn,
@@ -468,12 +496,12 @@ compatibility with @sc{matlab}.
                  F77_CHAR_ARG_LEN (1)
                  F77_CHAR_ARG_LEN (1)));
 
-      ComplexRowVector cwork (1 * nn);
+      ComplexRowVector cwork (nn);
 
       F77_XFCN (zhgeqz, ZHGEQZ,
                 (F77_CONST_CHAR_ARG2 (&qz_job, 1),
-                 F77_CONST_CHAR_ARG2 (&compq, 1),
-                 F77_CONST_CHAR_ARG2 (&compz, 1),
+                 F77_CONST_CHAR_ARG2 (&comp_q, 1),
+                 F77_CONST_CHAR_ARG2 (&comp_z, 1),
                  nn, ilo, ihi,
                  F77_DBLE_CMPLX_ARG (caa.fortran_vec ()), nn,
                  F77_DBLE_CMPLX_ARG (cbb.fortran_vec ()), nn,
@@ -481,12 +509,13 @@ compatibility with @sc{matlab}.
                  F77_DBLE_CMPLX_ARG (xbeta.fortran_vec ()),
                  F77_DBLE_CMPLX_ARG (CQ.fortran_vec ()), nn,
                  F77_DBLE_CMPLX_ARG (CZ.fortran_vec ()), nn,
-                 F77_DBLE_CMPLX_ARG (cwork.fortran_vec ()), nn, rwork.fortran_vec (), info
+                 F77_DBLE_CMPLX_ARG (cwork.fortran_vec ()), nn,
+                 rwork.fortran_vec (), info
                  F77_CHAR_ARG_LEN (1)
                  F77_CHAR_ARG_LEN (1)
                  F77_CHAR_ARG_LEN (1)));
 
-      if (compq == 'V')
+      if (comp_q == 'V')
         {
           // Left eigenvector.
           F77_XFCN (zggbak, ZGGBAK,
@@ -498,9 +527,9 @@ compatibility with @sc{matlab}.
                      F77_CHAR_ARG_LEN (1)));
         }
 
-      // Right eigenvector.
-      if (compz == 'V')
+      if (comp_z == 'V')
         {
+          // Right eigenvector.
           F77_XFCN (zggbak, ZGGBAK,
                     (F77_CONST_CHAR_ARG2 (&bal_job, 1),
                      F77_CONST_CHAR_ARG2 ("R", 1),
@@ -514,14 +543,14 @@ compatibility with @sc{matlab}.
   else
     {
 #if defined (DEBUG)
-      std::cout << "qz: peforming qr decomposition of bb" << std::endl;
+      std::cout << "qz: performing qr decomposition of bb" << std::endl;
 #endif
 
       // Compute the QR factorization of bb.
       octave::math::qr<Matrix> bqr (bb);
 
 #if defined (DEBUG)
-      std::cout << "qz: qr (bb) done; now peforming qz decomposition"
+      std::cout << "qz: qr (bb) done; now performing qz decomposition"
                 << std::endl;
 #endif
 
@@ -537,11 +566,11 @@ compatibility with @sc{matlab}.
       std::cout << "qz: updated aa " << std::endl;
       std::cout << "bqr.Q () = " << std::endl << bqr.Q () << std::endl;
 
-      if (compq == 'V')
+      if (comp_q == 'V')
         std::cout << "QQ =" << QQ << std::endl;
 #endif
 
-      if (compq == 'V')
+      if (comp_q == 'V')
         QQ = QQ * bqr.Q ();
 
 #if defined (DEBUG)
@@ -549,28 +578,28 @@ compatibility with @sc{matlab}.
 #endif
 
 #if defined (DEBUG)
-      std::cout << "qz: compq = " << compq << ", compz = " << compz
+      std::cout << "qz: comp_q = " << comp_q << ", comp_z = " << comp_z
                 << std::endl;
 #endif
 
       // Reduce to generalized Hessenberg form.
       F77_XFCN (dgghrd, DGGHRD,
-                (F77_CONST_CHAR_ARG2 (&compq, 1),
-                 F77_CONST_CHAR_ARG2 (&compz, 1),
+                (F77_CONST_CHAR_ARG2 (&comp_q, 1),
+                 F77_CONST_CHAR_ARG2 (&comp_z, 1),
                  nn, ilo, ihi, aa.fortran_vec (),
                  nn, bb.fortran_vec (), nn, QQ.fortran_vec (), nn,
                  ZZ.fortran_vec (), nn, info
                  F77_CHAR_ARG_LEN (1)
                  F77_CHAR_ARG_LEN (1)));
 
-      // Check if just computing generalized eigenvalues or if we're
-      // actually computing the decomposition.
+      // Check if just computing generalized eigenvalues,
+      // or if we're actually computing the decomposition.
 
       // Reduce to generalized Schur form.
       F77_XFCN (dhgeqz, DHGEQZ,
                 (F77_CONST_CHAR_ARG2 (&qz_job, 1),
-                 F77_CONST_CHAR_ARG2 (&compq, 1),
-                 F77_CONST_CHAR_ARG2 (&compz, 1),
+                 F77_CONST_CHAR_ARG2 (&comp_q, 1),
+                 F77_CONST_CHAR_ARG2 (&comp_z, 1),
                  nn, ilo, ihi, aa.fortran_vec (), nn, bb.fortran_vec (),
                  nn, alphar.fortran_vec (), alphai.fortran_vec (),
                  betar.fortran_vec (), QQ.fortran_vec (), nn,
@@ -579,7 +608,7 @@ compatibility with @sc{matlab}.
                  F77_CHAR_ARG_LEN (1)
                  F77_CHAR_ARG_LEN (1)));
 
-      if (compq == 'V')
+      if (comp_q == 'V')
         {
           F77_XFCN (dggbak, DGGBAK,
                     (F77_CONST_CHAR_ARG2 (&bal_job, 1),
@@ -590,14 +619,14 @@ compatibility with @sc{matlab}.
                      F77_CHAR_ARG_LEN (1)));
 
 #if defined (DEBUG)
-          if (compq == 'V')
+          if (comp_q == 'V')
             std::cout << "qz: balancing done; QQ=" << std::endl
                       << QQ << std::endl;
 #endif
         }
 
       // then right
-      if (compz == 'V')
+      if (comp_z == 'V')
         {
           F77_XFCN (dggbak, DGGBAK,
                     (F77_CONST_CHAR_ARG2 (&bal_job, 1),
@@ -608,7 +637,7 @@ compatibility with @sc{matlab}.
                      F77_CHAR_ARG_LEN (1)));
 
 #if defined (DEBUG)
-          if (compz == 'V')
+          if (comp_z == 'V')
             std::cout << "qz: balancing done; ZZ=" << std::endl
                       << ZZ << std::endl;
 #endif
@@ -617,11 +646,11 @@ compatibility with @sc{matlab}.
     }
 
   // Order the QZ decomposition?
-  if (! (ord_job == 'N' || ord_job == 'n'))
+  if (ord_job != 'N')
     {
       if (complex_case)
         // Probably not needed, but better be safe.
-        error ("qz: cannot re-order complex qz decomposition");
+        error ("qz: cannot re-order complex QZ decomposition");
 
 #if defined (DEBUG_SORT)
       std::cout << "qz: ordering eigenvalues: ord_job = "
@@ -630,17 +659,15 @@ compatibility with @sc{matlab}.
 
       // Declared static to avoid vfork/long jump compiler complaints.
       static sort_function sort_test;
-      sort_test = 0;
+      sort_test = nullptr;
 
       switch (ord_job)
         {
         case 'S':
-        case 's':
           sort_test = &fin;
           break;
 
         case 'B':
-        case 'b':
           sort_test = &fout;
           break;
 
@@ -653,13 +680,12 @@ compatibility with @sc{matlab}.
           break;
 
         default:
-          // Invalid order option (should never happen, since we
-          // checked the options at the top).
+          // Invalid order option
+          // (should never happen since options were checked at the top).
           panic_impossible ();
           break;
         }
 
-      octave_idx_type ndim, fail;
       double inf_norm;
 
       F77_XFCN (xdlange, XDLANGE,
@@ -674,7 +700,7 @@ compatibility with @sc{matlab}.
       octave_print_internal (std::cout, aa, 0);
       std::cout << std::endl << "bb="  << std::endl;
       octave_print_internal (std::cout, bb, 0);
-      if (compz == 'V')
+      if (comp_z == 'V')
         {
           std::cout << std::endl << "ZZ="  << std::endl;
           octave_print_internal (std::cout, ZZ, 0);
@@ -689,7 +715,9 @@ compatibility with @sc{matlab}.
       std::cout << std::endl;
 #endif
 
-      Array<octave_idx_type> ind (dim_vector (nn, 1));
+      Array<F77_INT> ind (dim_vector (nn, 1));
+
+      F77_INT ndim, fail;
 
       F77_XFCN (dsubsp, DSUBSP,
                 (nn, nn, aa.fortran_vec (), bb.fortran_vec (),
@@ -701,7 +729,7 @@ compatibility with @sc{matlab}.
       octave_print_internal (std::cout, aa, 0);
       std::cout << std::endl << "bb="  << std::endl;
       octave_print_internal (std::cout, bb, 0);
-      if (compz == 'V')
+      if (comp_z == 'V')
         {
           std::cout << std::endl << "ZZ="  << std::endl;
           octave_print_internal (std::cout, ZZ, 0);
@@ -710,21 +738,21 @@ compatibility with @sc{matlab}.
 #endif
 
       // Manually update alphar, alphai, betar.
-      static int jj;
+      static F77_INT j;
 
-      jj = 0;
-      while (jj < nn)
+      j = 0;
+      while (j < nn)
         {
 #if defined (DEBUG_EIG)
-          std::cout << "computing gen eig #" << jj << std::endl;
+          std::cout << "computing gen eig #" << j << std::endl;
 #endif
 
           // Number of zeros in this block.
-          static int zcnt;
+          static F77_INT zcnt;
 
-          if (jj == (nn-1))
+          if (j == (nn-1))
             zcnt = 1;
-          else if (aa(jj+1,jj) == 0)
+          else if (aa(j+1,j) == 0)
             zcnt = 1;
           else zcnt = 2;
 
@@ -733,16 +761,16 @@ compatibility with @sc{matlab}.
               // Real zero.
 #if defined (DEBUG_EIG)
               std::cout << "  single gen eig:" << std::endl;
-              std::cout << "  alphar(" << jj << ") = " << aa(jj,jj)
+              std::cout << "  alphar(" << j << ") = " << aa(j,j)
                         << std::endl;
-              std::cout << "  betar(" << jj << ") = " << bb(jj,jj)
+              std::cout << "  betar(" << j << ") = " << bb(j,j)
                         << std::endl;
-              std::cout << "  alphai(" << jj << ") = 0" << std::endl;
+              std::cout << "  alphai(" << j << ") = 0" << std::endl;
 #endif
 
-              alphar(jj) = aa(jj,jj);
-              alphai(jj) = 0;
-              betar(jj) = bb(jj,jj);
+              alphar(j) = aa(j,j);
+              alphai(j) = 0;
+              betar(j) = bb(j,j);
             }
           else
             {
@@ -753,24 +781,24 @@ compatibility with @sc{matlab}.
                         << setiosflags (std::ios::scientific)
                         << safmin << std::endl;
 
-              for (int idr = jj; idr <= jj+1; idr++)
+              for (F77_INT idr = j; idr <= j+1; idr++)
                 {
-                  for (int idc = jj; idc <= jj+1; idc++)
+                  for (F77_INT idc = j; idc <= j+1; idc++)
                     {
-                      std::cout << "aa(" << idr << "," << idc << ")="
+                      std::cout << "aa(" << idr << ',' << idc << ")="
                                 << aa(idr,idc) << std::endl;
-                      std::cout << "bb(" << idr << "," << idc << ")="
+                      std::cout << "bb(" << idr << ',' << idc << ")="
                                 << bb(idr,idc) << std::endl;
                     }
                 }
 #endif
 
               // FIXME: probably should be using
-              // fortran_vec instead of &aa(jj,jj) here.
+              // fortran_vec instead of &aa(j,j) here.
 
               double scale1, scale2, wr1, wr2, wi;
-              const double *aa_ptr = aa.data () + jj * nn + jj;
-              const double *bb_ptr = bb.data () + jj * nn + jj;
+              const double *aa_ptr = aa.data () + j * nn + j;
+              const double *bb_ptr = bb.data () + j * nn + j;
               F77_XFCN (dlag2, DLAG2,
                         (aa_ptr, nn, bb_ptr, nn, safmin,
                          scale1, scale2, wr1, wr2, wi));
@@ -785,23 +813,23 @@ compatibility with @sc{matlab}.
               // Just to be safe, check if it's a real pair.
               if (wi == 0)
                 {
-                  alphar(jj) = wr1;
-                  alphai(jj) = 0;
-                  betar(jj) = scale1;
-                  alphar(jj+1) = wr2;
-                  alphai(jj+1) = 0;
-                  betar(jj+1) = scale2;
+                  alphar(j) = wr1;
+                  alphai(j) = 0;
+                  betar(j) = scale1;
+                  alphar(j+1) = wr2;
+                  alphai(j+1) = 0;
+                  betar(j+1) = scale2;
                 }
               else
                 {
-                  alphar(jj) = alphar(jj+1) = wr1;
-                  alphai(jj) = -(alphai(jj+1) = wi);
-                  betar(jj)  = betar(jj+1) = scale1;
+                  alphar(j) = alphar(j+1) = wr1;
+                  alphai(j) = -(alphai(j+1) = wi);
+                  betar(j)  = betar(j+1) = scale1;
                 }
             }
 
           // Advance past this block.
-          jj += zcnt;
+          j += zcnt;
         }
 
 #if defined (DEBUG_SORT)
@@ -810,7 +838,7 @@ compatibility with @sc{matlab}.
       std::cout << std::endl << "bb="  << std::endl;
       octave_print_internal (std::cout, bb, 0);
 
-      if (compz == 'V')
+      if (comp_z == 'V')
         {
           std::cout << std::endl << "ZZ="  << std::endl;
           octave_print_internal (std::cout, ZZ, 0);
@@ -827,23 +855,17 @@ compatibility with @sc{matlab}.
 #endif
     }
 
-  // Compute generalized eigenvalues?
+  // Compute the generalized eigenvalues as well?
   ComplexColumnVector gev;
 
   if (nargout < 2 || nargout == 7 || (nargin == 3 && nargout == 4))
     {
       if (complex_case)
         {
-          int cnt = 0;
+          ComplexColumnVector tmp (nn);
 
-          for (int ii = 0; ii < nn; ii++)
-            cnt++;
-
-          ComplexColumnVector tmp (cnt);
-
-          cnt = 0;
-          for (int ii = 0; ii < nn; ii++)
-            tmp(cnt++) = xalpha(ii) / xbeta(ii);
+          for (F77_INT i = 0; i < nn; i++)
+            tmp(i) = xalpha(i) / xbeta(i);
 
           gev = tmp;
         }
@@ -854,18 +876,14 @@ compatibility with @sc{matlab}.
 #endif
 
           // Return finite generalized eigenvalues.
-          int cnt = 0;
+          ComplexColumnVector tmp (nn);
 
-          for (int ii = 0; ii < nn; ii++)
-            if (betar(ii) != 0)
-              cnt++;
+          F77_INT cnt = 0;
+          for (F77_INT i = 0; i < nn; i++)
+            if (betar(i) != 0)
+              tmp(cnt++) = Complex (alphar(i), alphai(i)) / betar(i);
 
-          ComplexColumnVector tmp (cnt);
-
-          cnt = 0;
-          for (int ii = 0; ii < nn; ii++)
-            if (betar(ii) != 0)
-              tmp(cnt++) = Complex(alphar(ii), alphai(ii))/betar(ii);
+          tmp.resize (cnt);  // Trim vector to number of return values
 
           gev = tmp;
         }
@@ -877,9 +895,9 @@ compatibility with @sc{matlab}.
       // Which side to compute?
       char side = (nargout == 5 ? 'R' : 'B');
       // Compute all of them and backtransform
-      char howmny = 'B';
+      char howmany = 'B';
       // Dummy pointer; select is not used.
-      octave_idx_type *select = 0;
+      F77_INT *select = nullptr;
 
       if (complex_case)
         {
@@ -887,16 +905,17 @@ compatibility with @sc{matlab}.
           CVR = CZ;
           ComplexRowVector cwork2 (2 * nn);
           RowVector rwork2 (8 * nn);
-          octave_idx_type m;
+          F77_INT m;
 
           F77_XFCN (ztgevc, ZTGEVC,
                     (F77_CONST_CHAR_ARG2 (&side, 1),
-                     F77_CONST_CHAR_ARG2 (&howmny, 1),
+                     F77_CONST_CHAR_ARG2 (&howmany, 1),
                      select, nn, F77_DBLE_CMPLX_ARG (caa.fortran_vec ()), nn,
                      F77_DBLE_CMPLX_ARG (cbb.fortran_vec ()),
                      nn, F77_DBLE_CMPLX_ARG (CVL.fortran_vec ()), nn,
                      F77_DBLE_CMPLX_ARG (CVR.fortran_vec ()), nn, nn,
-                     m, F77_DBLE_CMPLX_ARG (cwork2.fortran_vec ()), rwork2.fortran_vec (), info
+                     m, F77_DBLE_CMPLX_ARG (cwork2.fortran_vec ()),
+                     rwork2.fortran_vec (), info
                      F77_CHAR_ARG_LEN (1)
                      F77_CHAR_ARG_LEN (1)));
         }
@@ -908,11 +927,11 @@ compatibility with @sc{matlab}.
 
           VL = QQ;
           VR = ZZ;
-          octave_idx_type m;
+          F77_INT m;
 
           F77_XFCN (dtgevc, DTGEVC,
                     (F77_CONST_CHAR_ARG2 (&side, 1),
-                     F77_CONST_CHAR_ARG2 (&howmny, 1),
+                     F77_CONST_CHAR_ARG2 (&howmany, 1),
                      select, nn, aa.fortran_vec (), nn, bb.fortran_vec (),
                      nn, VL.fortran_vec (), nn, VR.fortran_vec (), nn, nn,
                      m, work.fortran_vec (), info
@@ -920,76 +939,81 @@ compatibility with @sc{matlab}.
                      F77_CHAR_ARG_LEN (1)));
 
           // Now construct the complex form of VV, WW.
-          int jj = 0;
+          F77_INT j = 0;
 
-          while (jj < nn)
+          while (j < nn)
             {
-              OCTAVE_QUIT;
+              octave_quit ();
 
               // See if real or complex eigenvalue.
 
               // Column increment; assume complex eigenvalue.
               int cinc = 2;
 
-              if (jj == (nn-1))
+              if (j == (nn-1))
                 // Single column.
                 cinc = 1;
-              else if (aa(jj+1,jj) == 0)
+              else if (aa(j+1,j) == 0)
                 cinc = 1;
 
               // Now copy the eigenvector (s) to CVR, CVL.
               if (cinc == 1)
                 {
-                  for (int ii = 0; ii < nn; ii++)
-                    CVR(ii,jj) = VR(ii,jj);
+                  for (F77_INT i = 0; i < nn; i++)
+                    CVR(i,j) = VR(i,j);
 
                   if (side == 'B')
-                    for (int ii = 0; ii < nn; ii++)
-                      CVL(ii,jj) = VL(ii,jj);
+                    for (F77_INT i = 0; i < nn; i++)
+                      CVL(i,j) = VL(i,j);
                 }
               else
                 {
                   // Double column; complex vector.
 
-                  for (int ii = 0; ii < nn; ii++)
+                  for (F77_INT i = 0; i < nn; i++)
                     {
-                      CVR(ii,jj) = Complex (VR(ii,jj), VR(ii,jj+1));
-                      CVR(ii,jj+1) = Complex (VR(ii,jj), -VR(ii,jj+1));
+                      CVR(i,j) = Complex (VR(i,j), VR(i,j+1));
+                      CVR(i,j+1) = Complex (VR(i,j), -VR(i,j+1));
                     }
 
                   if (side == 'B')
-                    for (int ii = 0; ii < nn; ii++)
+                    for (F77_INT i = 0; i < nn; i++)
                       {
-                        CVL(ii,jj) = Complex (VL(ii,jj), VL(ii,jj+1));
-                        CVL(ii,jj+1) = Complex (VL(ii,jj), -VL(ii,jj+1));
+                        CVL(i,j) = Complex (VL(i,j), VL(i,j+1));
+                        CVL(i,j+1) = Complex (VL(i,j), -VL(i,j+1));
                       }
                 }
 
               // Advance to next eigenvectors (if any).
-              jj += cinc;
+              j += cinc;
             }
         }
     }
 
+  octave_value_list retval (nargout);
+
   switch (nargout)
     {
     case 7:
       retval(6) = gev;
+      OCTAVE_FALLTHROUGH;
 
     case 6:
-      // Return eigenvectors.
+      // Return left eigenvectors.
       retval(5) = CVL;
+      OCTAVE_FALLTHROUGH;
 
     case 5:
-      // Return eigenvectors.
+      // Return right eigenvectors.
       retval(4) = CVR;
+      OCTAVE_FALLTHROUGH;
 
     case 4:
       if (nargin == 3)
         {
 #if defined (DEBUG)
           std::cout << "qz: sort: retval(3) = gev = " << std::endl;
-          octave_print_internal (std::cout, gev);
+          octave_print_internal (std::cout, ComplexMatrix (gev));
           std::cout << std::endl;
 #endif
           retval(3) = gev;
@@ -1001,6 +1025,7 @@ compatibility with @sc{matlab}.
           else
             retval(3) = ZZ;
         }
+      OCTAVE_FALLTHROUGH;
 
     case 3:
       if (nargin == 3)
@@ -1017,6 +1042,7 @@ compatibility with @sc{matlab}.
           else
             retval(2) = QQ.transpose ();
         }
+      OCTAVE_FALLTHROUGH;
 
     case 2:
       {
@@ -1058,7 +1084,7 @@ compatibility with @sc{matlab}.
     default:
       error ("qz: too many return arguments");
       break;
-  }
+    }
 
 #if defined (DEBUG)
   std::cout << "qz: exiting (at long last)" << std::endl;
@@ -1075,7 +1101,7 @@ compatibility with @sc{matlab}.
 %!assert (qz (a,b), 1)
 %!assert (isempty (qz (a,c)))
 
-## Exaple 7.7.3 in Golub & Van Loan
+## Example 7.7.3 in Golub & Van Loan
 %!test
 %! a = [ 10  1  2;
 %!        1  2 -1;
@@ -1094,7 +1120,6 @@ compatibility with @sc{matlab}.
 %! A = [0, 0, -1, 0; 1, 0, 0, 0; -1, 0, -2, -1; 0, -1, 1, 0];
 %! B = [0, 0, 0, 0; 0, 1, 0, 0; 0, 0, 1, 0; 0, 0, 0, 1];
 %! [AA, BB, Q, Z1] = qz (A, B);
-%! [AA, BB, Z2] = qz (A, B, '-');
+%! [AA, BB, Z2] = qz (A, B, "-");
 %! assert (Z1, Z2);
 */
-
